import type { RoutedWindow } from '@goldenwere/mackenzii-types'

import type { AttributeObserver } from './observer'

export type LinkEmbedAttributes = typeof LinkEmbed.observedAttributes[number]

declare const window: RoutedWindow

/**
 * A link embed is a link with a title and an optional icon + subtitle
 */
export class LinkEmbed extends HTMLElement {
  static observedAttributes = [
    'icon',
    'title',
    'subtitle',
    'href',
    'target',
  ] as const

  _observers: AttributeObserver<LinkEmbedAttributes> = {} as any

  constructor() {
    super()

    const { ownerDocument } = this

    const _link = ownerDocument.createElement('a')
    _link.setAttribute('rel', 'noreferrer noopener')
    _link.addEventListener('click', (e) => {
      if (_link.dataset?.href?.startsWith('/') && !!window.router) {
        e.preventDefault()
        const route = window.router.resolve({ path: _link.dataset.href })
        if (route.path !== window.router.currentRoute.value.path) {
          window.router.push(route)
        }
      }
    })
    this._observers.href = {
      element: _link,
      observer: (oldVal, newVal: string) => {
        const { element } = this._observers.href
        element!.setAttribute('href', newVal)
        element!.dataset.href = newVal
        if (!!window.router) {
          const route = window.router.resolve({ path: newVal })
          if (route.path === window.router.currentRoute.value.path) {
            element!.classList.add('current-route')
          }
        }
      },
    }
    this._observers.target = {
      element: _link,
      observer: (oldVal, newVal) => {
        const { element } = this._observers.href
        if (!!newVal) {
          element!.setAttribute('target', newVal)
        } else {
          element!.removeAttribute('target')
        }
      },
    }
    this.append(_link)

    const _iconWrapper = ownerDocument.createElement('div')
    _iconWrapper.classList.add('icon-wrapper')
    _link.append(_iconWrapper)

    const _icon = ownerDocument.createElement('img')
    _icon.alt = '' // decorative
    _icon.classList.add('no-inflate')
    _iconWrapper.append(_icon)
    this._observers.icon = {
      element: _icon,
      observer: (oldVal, newVal) => {
        if (!!newVal) {
          _icon.setAttribute('src', newVal)
          _iconWrapper.style.display = 'inherit'
        } else {
          _iconWrapper.style.display = 'hidden'
        }
      },
    }

    const _textWrapper = ownerDocument.createElement('div')
    _textWrapper.classList.add('text')
    _link.append(_textWrapper)

    const _title = ownerDocument.createElement('p')
    this._observers.title = {
      element: _title,
      observer: (oldVal, newVal) => {
        const { element } = this._observers.title
        element!.innerHTML = newVal
      },
    }
    
    const _subtitle = ownerDocument.createElement('p')
    if (!this.getAttribute('subtitle')) {
      _subtitle.style.display = 'hidden'
    }
    this._observers.subtitle = {
      element: _subtitle,
      observer: (oldVal, newVal) => {
        const { element } = this._observers.subtitle
        if (!!newVal) {
          element!.innerHTML = newVal
          if (_subtitle.parentElement !== _textWrapper) {
            _textWrapper.append(_subtitle)
          }
        } else if (_subtitle.parentElement === _textWrapper) {
          _textWrapper.removeChild(_subtitle)
        }
      },
    }

    _textWrapper.append(_title)
  }

  attributeChangedCallback(name: LinkEmbedAttributes, oldVal: any, newVal: any) {
    this._observers[name].observer(oldVal, newVal)
  }
}

export const registerLinkEmbed = () => {
  if (!customElements.get('link-embed')) {
    customElements.define('link-embed', LinkEmbed)
  }
}
